<?php

namespace FiloBlu\Refilo\Model\Indexer\Core;

use FiloBlu\Refilo\Model\Adapter\Mongo;
use FiloBlu\Refilo\Model\Indexer\Core\Provider\QueryProviderInterface;
use FiloBlu\Refilo\Model\Indexer\Core\Provider\QueryProviderInterfaceFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Mview\ActionInterface;

abstract class AbstractMongoIndexer implements \Magento\Framework\Indexer\ActionInterface, ActionInterface
{
    /**
     * @var SqlToMongoAdapterInterface
     */
    protected $sqlToMongoAdapter;
    /**
     * @var Mongo
     */
    protected $mongo;
    /**
     * @var QueryProviderInterfaceFactory
     */
    protected $queryProviderFactory;

    /**
     * @var
     */
    protected $getDatabaseConfiguration;
    /**
     * @var string
     */
    private $databaseConfigurationKey;
    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * AbstractMongoIndexer constructor.
     * @param SqlToMongoAdapterInterface $sqlToMongoAdapter
     * @param QueryProviderInterfaceFactory $queryProviderFactory
     * @param ScopeConfigInterface $scopeConfig
     * @param Mongo $mongo
     * @param string $databaseConfigurationKey
     */
    public function __construct(
        SqlToMongoAdapterInterface $sqlToMongoAdapter,
        QueryProviderInterfaceFactory $queryProviderFactory,
        ScopeConfigInterface $scopeConfig,
        Mongo $mongo,
        string $databaseConfigurationKey
    ) {
        $this->sqlToMongoAdapter = $sqlToMongoAdapter;
        $this->mongo = $mongo;
        $this->queryProviderFactory = $queryProviderFactory;
        $this->databaseConfigurationKey = $databaseConfigurationKey;
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * @inheritDoc
     */
    public function executeFull()
    {
        $this->performAction();
    }

    protected function performAction($ids = null)
    {
        /** @var QueryProviderInterface $provider */
        $provider = $this->queryProviderFactory->create();
        $provider->filter($ids);

        $result = $this->sqlToMongoAdapter->adaptFromSql($provider);
        $this->mongo->update(
            $this->getMongoKey(),
            $result,
            $this->getTargetMongoCollection(),
            $this->getInsertMode(),
            $this->getMongoDb()
        );
    }

    /**
     * @return string
     */
    abstract public function getMongoKey(): string;

    /**
     * @return string
     */
    abstract public function getTargetMongoCollection(): string;

    /**
     * @return bool
     */
    abstract public function getInsertMode(): bool;

    /**
     * @inheritDoc
     */
    public function executeRow($id)
    {
        $this->executeList([$id]);
    }

    /**
     * @inheritDoc
     */
    public function executeList(array $ids)
    {
        $this->performAction($ids);
    }

    /**
     * @inheritDoc
     */
    public function execute($ids)
    {
        $this->executeList($ids);
    }

    /**
     * @return mixed
     */
    public function getMongoDb()
    {
        return $this->scopeConfig($this->getDatabaseConfiguration);
    }
}
